<?php
/**
 *    This file is part of "PCPIN Chat 6".
 *
 *    "PCPIN Chat 6" is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    "PCPIN Chat 6" is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Class PCPIN_Version
 * Manage version informations
 * @author Konstantin Reznichak <k.reznichak@pcpin.com>
 * @copyright Copyright &copy; 2007, Konstantin Reznichak
 */
class PCPIN_Version extends PCPIN_Session {

  /**
   * Current version number
   * @var   float
   */
  var $version=0.0;

  /**
   * A key generated by chat in order to secure new version check.
   * This key will be transferred to the version check server.
   * A valid answer from version check server MUST contain the same key.
   * @var   string
   */
  var $version_check_key='';

  /**
   * Date of the last version check (MySQL DATETIME)
   * @var   string
   */
  var $last_version_check='';

  /**
   * If new version is available: new version number
   * @var   float
   */
  var $new_version_available=0.0;

  /**
   * Download URL of the newest available version
   * @var   string
   */
  var $new_version_url='';




  /**
   * Constructor
   * @param   object  &$sessionhandler  Session handler
   */
  function PCPIN_Version(&$sessionhandler) {
    // Init object
    $this->_s_init($sessionhandler, $this);
  }

  /**
   * Update version number
   * @param   string  $version  New version number
   * @return  boolean   TRUE on success or FALSE on error
   */
  function setVersion($version=''){
    $result=false;
    $version=trim($version);
    if (!empty($version)) {
      $query=$this->_db_makeQuery(1700, $version);
      if ($result=$this->_db_query($query)) {
        $this->_db_freeResult($result);
        $result=true;
      }
    }
    return $result;
  }

  /**
   * Update last version check time
   * @return  boolean   TRUE on success or FALSE on error
   */
  function setLastVersionCheckTime() {
    $query=$this->_db_makeQuery(1700, null, date('Y-m-d H:i:s'));
    if ($result=$this->_db_query($query)) {
      $this->_db_freeResult($result);
      $result=true;
    }
    return $result;
  }

  /**
   * Update newest available version number
   * @param   string  $version    Version number
   * @return  boolean   TRUE on success or FALSE on error
   */
  function setNewestAvailableVersion($version='') {
    $result=false;
    $version=trim($version);
    if (!empty($version)) {
      $query=$this->_db_makeQuery(1700, null, null, $version);
      if ($result=$this->_db_query($query)) {
        $this->_db_freeResult($result);
        $result=true;
      }
    }
    return $result;
  }

  /**
   * Update version check security key
   * @param   string  $key    Security key
   * @return  boolean   TRUE on success or FALSE on error
   */
  function setVersionCheckKey($key='') {
    if (!empty($key)) {
      $key=md5($key);
    } else {
      $key='-blank-';
    }
    $query=$this->_db_makeQuery(1700, null, null, null, $key);
    if ($result=$this->_db_query($query)) {
      $this->_db_freeResult($result);
      $result=true;
    }
    return $result;
  }

  /**
   * Save download URL of the latest available version
   * @param   string  $url    URL
   * @return  boolean   TRUE on success or FALSE on error
   */
  function setNewVersionDownloadUrl($url='') {
    $result=false;
    $url=trim($url);
    if (!empty($url)) {
      $query=$this->_db_makeQuery(1700, null, null, null, null, $url);
      if ($result=$this->_db_query($query)) {
        $this->_db_freeResult($result);
        $result=true;
      }
    }
    return $result;
  }


}
?>